/** \file valueset.h
 *	Set of values
 *
 * $Id: valueset.h,v 1.7 2002/02/20 18:05:11 lecroart Exp $
 */

/* Copyright, 2000 Nevrax Ltd.
 *
 * This file is part of NEVRAX NEL.
 * NEVRAX NEL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.

 * NEVRAX NEL is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with NEVRAX NEL; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

#ifndef NL_VALUE_SET_H
#define NL_VALUE_SET_H

#include "nel/ai/agent/agent.h"

namespace NLAILOGIC {

	class IBaseVar;
	class CClause;

	class CValueSet  : public NLAIAGENT::IObjetOp
	{
		protected:
			NLAIAGENT::IObjectIA **_Values;
			sint32 _NbValues;

		public: 
			CValueSet();
			CValueSet(sint32);
			CValueSet(const CValueSet &);
			CValueSet(std::vector<IBaseVar *> &);
			CValueSet(CValueSet *,std::vector<sint32> &);
			CValueSet(sint32,std::list<NLAIAGENT::IObjectIA *> *,std::vector<sint32> &);
			virtual ~CValueSet();
			virtual NLAIAGENT::IObjectIA *getValue(sint32);
			void setValue(sint32, NLAIAGENT::IObjectIA *);
			CValueSet *unify(const CValueSet *) const ;
			CValueSet *unify(std::list<NLAIAGENT::IObjectIA *> *vals, std::vector<sint32> &pos_vals) const;
			CValueSet *unify(CValueSet *vals, std::vector<sint32> &pos_vals) const;
			bool operator==(CValueSet *);
			virtual NLAIAGENT::IObjectIA *operator[](sint32);
			sint32 size();
			sint32 undefined() const;
			std::list<NLAIAGENT::IObjectIA *> *getValues();
			virtual void getDebugString(std::string &) const;

			virtual const NLAIC::IBasicType *clone() const;
			virtual const NLAIC::IBasicType *newInstance() const;
			virtual const NLAIC::CIdentType &getType() const;
			virtual void save(NLMISC::IStream &os);
			virtual void load(NLMISC::IStream &is);

			static const NLAIC::CIdentType IdValueSet;

			virtual NLAIAGENT::IObjetOp *operator ! () const;
			virtual NLAIAGENT::IObjectIA *operator != (NLAIAGENT::IObjectIA &a) const;
			virtual NLAIAGENT::IObjectIA *operator == (NLAIAGENT::IObjectIA &a) const;

			virtual bool isEqual(const NLAIAGENT::IBasicObjectIA &) const;
			

			virtual bool isTrue() const;

			static const NLAIAGENT::TProcessStatement state;

			virtual const IObjectIA::CProcessResult &run();
			CValueSet *forward(CClause *,std::vector<sint32> &);

			virtual void setSize(sint32);
	};
}

#endif
