/** \file main.cpp
 * mini agent exemple
 *
 * $Id: main.cpp,v 1.4 2002/04/16 08:03:50 chafik Exp $
 */

/* Copyright, 2001 Nevrax Ltd.
 *
 * This file is part of NEVRAX NEL.
 * NEVRAX NEL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.

 * NEVRAX NEL is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with NEVRAX NEL; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

#include "main.h"

//For init lib
#include "nel/ai/nl_ai.h"


namespace Expl
{

	//Type of the class
	const NLAIC::CIdentType &CIO::getType() const
	{
		//register the class in the classFactory.
		static const NLAIC::CIdentType idCIO("CIO",	NLAIC::CSelfClassFactory(*this),
													NLAIC::CTypeOfObject(NLAIC::CTypeOfObject::tObject),NLAIC::CTypeOfOperator(0));
		return idCIO;

	}

	//we could do better, but time .....
	void CIO::Echo(char *txt, ...) const
	{
		//Large to be sure
		char lageBuffer[64*1024];
		char	temp[64*1024];
		va_list argument;
		va_start (argument, txt);	
		
		vsprintf(temp, txt, argument);		
		strcpy(lageBuffer,temp);
		
		nlinfo("%s",lageBuffer);		
	}


	CAgentManager *CAgentService::Agent = NULL;
	NLMISC::CPath *CAgentService::Path = NULL;
	std::list<std::string> CAgentService::AgScript = std::list<std::string>();
	CIO *CAgentService::AgIO = new CIO();


	CAgentService::CAgentService()
	{
	}

	void CAgentService::init()
	{
		NLAILINK::setLocalServerID((uint8)getServiceId());
		//This for uinitialize the ai lib
		NLAILINK::initIALib();
		//This for uinitialize the manager how deliver distruibuted message.
		NLAILINK::setMainManager(CAgentService::Agent);
		//This time is allow message transfer.
		setUpdateTimeout(10);

		CAgentService::Path = new NLMISC::CPath;

		//Set the input/output way.
		NLAIC::setDefaultIIO(CAgentService::AgIO);

		NLMISC::CConfigFile cf;
		try
		{
			//read the special agent var in the ag_s.cfg file.
			cf.load ("ag_s.cfg");
			NLMISC::CConfigFile::CVar &ScriptPath = cf.getVar ("ScriptPath");
			sint i;
			for(i = 0; i < ScriptPath.size(); i ++)
			{
				std::string s = ScriptPath.asString(i);
				CAgentService::Path->addSearchPath(s, true,true);				
			}

			NLMISC::CConfigFile::CVar &RunScript = cf.getVar ("RunScript");			
			for(i = 0; i < RunScript.size(); i ++)
			{
				std::string s = RunScript.asString(i);
				AgScript.push_back(std::string(s));
			}			
		}
		catch (NLMISC::EConfigFile &e)
		{
			nlerror("%s",e.what());
		}

		//instance of the agent manager.
		CAgentService::Agent = new CAgentManager(CAgentService::AgIO);

		//Script run if exist.
		CAgentService::Agent->init();
	}

	void CAgentService::release()
	{
		Agent->release();		
	}

	bool CAgentService::update()
	{
		Agent->run();
		//This lines is to run the agent internal timer.
		{
			NLMISC::CSynchronized<NLAIAGENT::CAgentScript *>::CAccessor accessor(NLAIAGENT::CAgentManagerTimer::TimerManager);
			accessor.value()->run();
		}
		return true;
	}


	/**
	 * Callback that processes a message aimed at an agent
	 *
	 * Message expected : AGT
	 */
	void CAgentService::cbProcessAgentMessage(NLNET::CMessage& agtinmsg, const std::string &name, uint16 sid )
	{	
		// Agent header
		std::string msgname;

		///The sid of the agent
		NLAIAGENT::CAgentNumber dest_aid;
		agtinmsg.serial( dest_aid );		
		
		NLAIAGENT::IRefrence *ag;
		if ( (ag = Agent->agentIsPresent( dest_aid )) != NULL )
		{		
			Agent->deliverMsg( ag, agtinmsg );
		}
		else
		{

			throw CExceptionSidError();	
		}
	}
	
}

NLNET::TUnifiedCallbackItem CallbackArray [] =
{		
	///AGT is the name of the message of the NLNET message who it have the distributed agent message.
	{ "AGT", Expl::CAgentService::cbProcessAgentMessage }	
};

NLNET_SERVICE_MAIN( Expl::CAgentService, AgentServiceName, "minAg_s", 0, CallbackArray, ".", "." );

